#include "skein/particlemap.h"
#include "skein/particle.h"

const Particle& ParticleMap::getParticle(const std::string& id) const
{
    return _particles.at(id);
}

const Particle& ParticleMap::getParent(const std::string& childId) const
{
    const std::string parentId = _relationships.at(childId);
    return _particles.at(parentId);
}

const Orbit& ParticleMap::getOrbit(const std::string& id) const
{
    return _orbits.at(id);
}

glm::dvec3 ParticleMap::getParticlePosition(const std::string& id, double time) const
{
    if (_orbits.find(id) == _orbits.end())
        return {0,0,0};

    const Particle* particle = &(_particles.at(id));
    glm::dvec3 pos(0,0,0);

    do
    {
        const std::string& id = particle->getId();
        const std::string& parentId = _relationships.at(id);
        const Particle& parent = _particles.at(parentId);
        const double gravitationalParameter = parent.getGravitationalParameter();
        const Orbit& orbit = _orbits.at(id);

        pos += orbit.getPosition(gravitationalParameter, time);

        auto it = _relationships.find(parentId);
        if (it != _relationships.end())
        {
            particle = &parent;
        }
        else
        {
            particle = nullptr;
        }
    } while (particle != nullptr);

    return pos;
}

void ParticleMap::setParticle(const Particle& particle)
{
    _particles.insert({particle.getId(), particle});
}

void ParticleMap::setRelationship(const std::string& parentId, const std::string& childId, const Orbit& orbit)
{
    // map children to parent - children can only have one parent, so we use the map to
    // identify the parent directly using the child as the key
    _relationships.insert({childId, parentId});
    _orbits.insert({childId, orbit});
}
